@@ -1,19 +1,79 @@
 use proc_macro2::TokenStream;
 use quote::quote;
 
-use crate::input::Input;
+use crate::input::{Input, TokenStreamIterator};
+use crate::names;
 
 pub fn derive(input: &Input) -> TokenStream {
-    let vec_name = &input.vec_name();
-    let slice_name = &input.slice_name();
-    let slice_mut_name = &input.slice_mut_name();
-    let ref_name = &input.ref_name();
-    let ref_mut_name = &input.ref_mut_name();
+    let vec_name = names::vec_name(&input.name);
+    let slice_name = names::slice_name(&input.name);
+    let slice_mut_name = names::slice_mut_name(&input.name);
+    let ref_name = names::ref_name(&input.name);
+    let ref_mut_name = names::ref_mut_name(&input.name);
     let fields_names = input.fields.iter()
                                    .map(|field| field.ident.clone().unwrap())
                                    .collect::<Vec<_>>();
-    let fields_names_1 = &fields_names;
-    let fields_names_2 = &fields_names;
+
+    let get_unchecked = input.iter_fields().map(
+        |(field_ident, _, is_nested)| {
+            if is_nested {
+                quote! {
+                    #field_ident: self.clone().get_unchecked(slice.#field_ident),
+                }
+            }
+            else {
+                quote! {
+                    #field_ident: slice.#field_ident.get_unchecked(self.clone()),
+                }
+            }
+        },
+    ).concat();
+
+    let get_unchecked_mut = input.iter_fields().map(
+        |(field_ident, _, is_nested)| {
+            if is_nested {
+                quote! {
+                    #field_ident: self.clone().get_unchecked_mut(slice.#field_ident),
+                }
+            }
+            else {
+                quote! {
+                    #field_ident: slice.#field_ident.get_unchecked_mut(self.clone()),
+                }
+            }
+        },
+    ).concat();
+
+    let index = input.iter_fields().map(
+        |(field_ident, _, is_nested)| {
+            if is_nested {
+                quote! {
+                    #field_ident: self.clone().index(slice.#field_ident),
+                }
+            }
+            else {
+                quote! {
+                    #field_ident: & slice.#field_ident[self.clone()],
+                }
+            }
+        },
+    ).concat();
+
+    let index_mut = input.iter_fields().map(
+        |(field_ident, _, is_nested)| {
+            if is_nested {
+                quote! {
+                    #field_ident: self.clone().index_mut(slice.#field_ident),
+                }
+            }
+            else {
+                quote! {
+                    #field_ident: &mut slice.#field_ident[self.clone()],
+                }
+            }
+        },
+    ).concat();
+
     let first_field_name = &fields_names[0];
 
     quote!{
@@ -32,16 +92,12 @@ pub fn derive(input: &Input) -> TokenStream {
 
             #[inline]
             unsafe fn get_unchecked(self, soa: &'a #vec_name) -> Self::RefOutput {
-                #ref_name {
-                    #(#fields_names_1: soa.#fields_names_2.get_unchecked(self),)*
-                }
+                self.get_unchecked(soa.as_slice())
             }
 
             #[inline]
             fn index(self, soa: &'a #vec_name) -> Self::RefOutput {
-                #ref_name {
-                    #(#fields_names_1: & soa.#fields_names_2[self],)*
-                }
+                self.index(soa.as_slice())
             }
         }
 
@@ -59,16 +115,12 @@ pub fn derive(input: &Input) -> TokenStream {
 
             #[inline]
             unsafe fn get_unchecked_mut(self, soa: &'a mut #vec_name) -> Self::MutOutput {
-                #ref_mut_name {
-                    #(#fields_names_1: soa.#fields_names_2.get_unchecked_mut(self),)*
-                }
+                self.get_unchecked_mut(soa.as_mut_slice())
             }
 
             #[inline]
             fn index_mut(self, soa: &'a mut #vec_name) -> Self::MutOutput {
-                #ref_mut_name {
-                    #(#fields_names_1: &mut soa.#fields_names_2[self],)*
-                }
+                self.index_mut(soa.as_mut_slice())
             }
         }
 
@@ -89,16 +141,12 @@ pub fn derive(input: &Input) -> TokenStream {
 
             #[inline]
             unsafe fn get_unchecked(self, soa: &'a #vec_name) -> Self::RefOutput {
-                #slice_name {
-                    #(#fields_names_1: soa.#fields_names_2.get_unchecked(self.clone()),)*
-                }
+                self.get_unchecked(soa.as_slice())
             }
 
             #[inline]
             fn index(self, soa: &'a #vec_name) -> Self::RefOutput {
-                #slice_name {
-                    #(#fields_names_1: & soa.#fields_names_2[self.clone()],)*
-                }
+                self.index(soa.as_slice())
             }
         }
 
@@ -116,16 +164,12 @@ pub fn derive(input: &Input) -> TokenStream {
 
             #[inline]
             unsafe fn get_unchecked_mut(self, soa: &'a mut #vec_name) -> Self::MutOutput {
-                #slice_mut_name {
-                    #(#fields_names_1: soa.#fields_names_2.get_unchecked_mut(self.clone()),)*
-                }
+                self.get_unchecked_mut(soa.as_mut_slice())
             }
 
             #[inline]
             fn index_mut(self, soa: &'a mut #vec_name) -> Self::MutOutput {
-                #slice_mut_name {
-                    #(#fields_names_1: &mut soa.#fields_names_2[self.clone()],)*
-                }
+                self.index_mut(soa.as_mut_slice())
             }
         }
 
@@ -354,14 +398,14 @@ pub fn derive(input: &Input) -> TokenStream {
             #[inline]
             unsafe fn get_unchecked(self, slice: #slice_name<'a>) -> Self::RefOutput {
                 #ref_name {
-                    #(#fields_names_1: slice.#fields_names_2.get_unchecked(self),)*
+                    #get_unchecked
                 }
             }
 
             #[inline]
             fn index(self, slice: #slice_name<'a>) -> Self::RefOutput {
                 #ref_name {
-                    #(#fields_names_1: & slice.#fields_names_2[self],)*
+                    #index
                 }
             }
         }
@@ -381,14 +425,14 @@ pub fn derive(input: &Input) -> TokenStream {
             #[inline]
             unsafe fn get_unchecked_mut(self, slice: #slice_mut_name<'a>) -> Self::MutOutput {
                 #ref_mut_name {
-                    #(#fields_names_1: slice.#fields_names_2.get_unchecked_mut(self),)*
+                    #get_unchecked_mut
                 }
             }
 
             #[inline]
             fn index_mut(self, slice: #slice_mut_name<'a>) -> Self::MutOutput {
                 #ref_mut_name {
-                    #(#fields_names_1: &mut slice.#fields_names_2[self],)*
+                    #index_mut
                 }
             }
         }
@@ -411,14 +455,14 @@ pub fn derive(input: &Input) -> TokenStream {
             #[inline]
             unsafe fn get_unchecked(self, slice: #slice_name<'a>) -> Self::RefOutput {
                 #slice_name {
-                    #(#fields_names_1: slice.#fields_names_2.get_unchecked(self.clone()),)*
+                    #get_unchecked
                 }
             }
 
             #[inline]
             fn index(self, slice: #slice_name<'a>) -> Self::RefOutput {
                 #slice_name {
-                    #(#fields_names_1: & slice.#fields_names_2[self.clone()],)*
+                    #index
                 }
             }
         }
@@ -438,14 +482,14 @@ pub fn derive(input: &Input) -> TokenStream {
             #[inline]
             unsafe fn get_unchecked_mut(self, slice: #slice_mut_name<'a>) -> Self::MutOutput {
                 #slice_mut_name {
-                    #(#fields_names_1: slice.#fields_names_2.get_unchecked_mut(self.clone()),)*
+                    #get_unchecked_mut
                 }
             }
 
             #[inline]
             fn index_mut(self, slice: #slice_mut_name<'a>) -> Self::MutOutput {
                 #slice_mut_name {
-                    #(#fields_names_1: &mut slice.#fields_names_2[self.clone()],)*
+                    #index_mut
                 }
             }
         }
