@@ -1,15 +1,15 @@
 #![warn(clippy::all, clippy::pedantic)]
+
 #![allow(clippy::needless_return, clippy::redundant_field_names)]
 #![allow(clippy::use_self, clippy::too_many_lines, clippy::missing_panics_doc)]
-// TODO: improve the code and make it simpler to read
-#![allow(clippy::cognitive_complexity)]
 
 extern crate proc_macro;
 
-use proc_macro2::TokenStream;
+use proc_macro2::{TokenStream};
 use quote::TokenStreamExt;
 
 mod index;
+#[macro_use]
 mod input;
 mod iter;
 mod ptr;
@@ -17,7 +17,9 @@ mod refs;
 mod slice;
 mod vec;
 
-#[proc_macro_derive(StructOfArray, attributes(soa_derive))]
+pub(crate) mod names;
+
+#[proc_macro_derive(StructOfArray, attributes(soa_derive, soa_attr, nested_soa))]
 pub fn soa_derive(input: proc_macro::TokenStream) -> proc_macro::TokenStream {
     let ast = syn::parse(input).unwrap();
     let input = input::Input::new(ast);
@@ -38,7 +40,7 @@ use crate::input::Input;
 use quote::quote;
 fn derive_trait(input: &Input) -> TokenStream {
     let name = &input.name;
-    let vec_name = &input.vec_name();
+    let vec_name = names::vec_name(name);
 
     quote! {
         impl soa_derive::StructOfArray for #name {
